//
//  MasterViewController.m
//  TestDataManageer
//
//  Created by Akabeko on 11/11/05.
//  Copyright (c) 2011, Akabeko. All rights reserved.
//

#import "BooksViewController.h"
#import "DaoBooks.h"
#import "Book.h"

@interface BooksViewController()
@property (nonatomic, retain) DaoBooks*            daoBooks; //! 書籍リストを管理するオブジェクト
@property (nonatomic, retain) NSMutableArray*      authors;  //! 著者名コレクション
@property (nonatomic, retain) NSMutableDictionary* books;    //! 著者名をキーとする書籍情報ディクショナリ

- (void)addBook:(id)sender;
- (void)addNewBook:(Book*)newBook;
- (Book*)bookAtIndexPath:(NSIndexPath*)indexPath;
- (void)removeBook:(NSIndexPath*)indexPath;
- (void)removeOldBook:(Book*)oldBook;
@end

@implementation BooksViewController

@synthesize daoBooks, authors, books;

#pragma mark - Lifecycle methods

/**
 * View が読み込まれる時に発生します
 */
- (void)viewDidLoad
{
	[super viewDidLoad];
	
	self.daoBooks = [[[DaoBooks alloc] init] autorelease];
	self.authors  = [[[NSMutableArray alloc] initWithCapacity:0] autorelease];
	self.books    = [[[NSMutableDictionary alloc] initWithCapacity:0] autorelease];
	
	NSArray* existBooks = [self.daoBooks books];
	for( Book* book in existBooks ) { [self addNewBook:book]; }
	
	self.title = NSLocalizedString( @"BOOK_LIST_TITLE", @"" );
	
	self.navigationController.navigationBar.barStyle = UIBarStyleBlack;
	
	// 編集ボタン
	self.navigationItem.leftBarButtonItem = self.editButtonItem;
	
	// 追加ボタン
    UIBarButtonItem* addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector( addBook: )];
    self.navigationItem.rightBarButtonItem = addButton;
    [addButton release];
}

/**
 * View が破棄される時に発生します。
 */
- (void)viewDidUnload
{
	self.daoBooks = nil;
	self.authors  = nil;
	self.books    = nil;
	
	[super viewDidUnload];
}

/**
 * メモリを解放します。
 */
- (void)dealloc
{
 	self.daoBooks = nil;
	self.authors  = nil;
	self.books    = nil;
	
	[super dealloc];
}

#pragma mark - UITableViewController methods

/**
 * テーブルのセクション数を取得します。
 *
 * @param tableView テーブル。
 *
 * @return セクション数。
 */
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
	return self.authors.count;
}

/**
 * テーブルの指定セクションにおける行数を取得します。
 *
 * @param tableView テーブル。
 * @param section   セクション。
 *
 * @return 行数。
 */
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
	NSArray* booksByAuthor = [self.books objectForKey:[self.authors objectAtIndex:section]];
	return booksByAuthor.count;
}

/**
 * セクションのタイトルを取得します。
 *
 * @param tableView テーブル。
 * @param section   セクション。
 *
 * @return タイトル。
 */
- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
	return [self.authors objectAtIndex:section];
}

/**
 * 指定位置のセルを取得します。
 *
 * @param tableView テーブル。
 * @param indexPath 位置。
 *
 * @return セル。
 */
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	static NSString* CellIdentifier = @"Cell";
    
	UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
	if( cell == nil )
	{
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
		cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
	}
	
	Book* book = [self bookAtIndexPath:indexPath];
	cell.textLabel.text = book.title;
	
	return cell;
}

/**
 * セルが選択された時に発生します。
 *
 * @param tableView テーブル。
 * @param indexPath 位置。
 */
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	EditBookController* editor = [[EditBookController alloc] initWithNibName:@"EditBook" bundle:nil];
	editor.delegate = self;
	editor.book     = [self bookAtIndexPath:indexPath];
	editor.title    = NSLocalizedString( @"BOOK_EDIT_TITLE", @"" );
	
	[self.navigationController pushViewController:editor animated:YES];
	[editor release];
}

/**
 * テーブルの編集を反映させる時に発生します。
 *
 * @param tableView    テーブル。
 * @param editingStyle 編集方法。
 * @param indexPath    編集個所。
 */
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
	if( editingStyle == UITableViewCellEditingStyleDelete )
	{
		[self removeBook:indexPath];
	}
}

#pragma mark - EditBookDelegate methods

/**
 * 書籍の追加が完了した時に発生します。
 *
 * @param newBook 追加された書籍。キャンセルされた場合は nil。
 */
- (void)addBookDidFinish:(Book *)newBook
{
	if( newBook )
	{
		[self addNewBook:newBook];
		[self.daoBooks add:newBook];
		[self.tableView reloadData];
	}
	
	[self.navigationController dismissModalViewControllerAnimated:YES];
}

/**
 * 書籍の編集が完了した時に発生します。
 *
 * @param oldBook 編集前の書籍。
 * @param newBook 編集後の書籍。
 */
- (void)editBookDidFinish:(Book *)oldBook newBook:(Book *)newBook
{
	if( [oldBook.author isEqualToString:newBook.author] )
	{
		NSMutableArray* booksByAuthor = [self.books objectForKey:newBook.author];
		for( Book* book in booksByAuthor )
		{
			if( book.bookId == oldBook.bookId )
			{
				book.title     = newBook.title;
				book.copyright = newBook.copyright;
				[self.daoBooks update:newBook];
				break;
			}
		}
		
	}
	else
	{
		[self removeOldBook:oldBook];
		[self addNewBook:newBook];
		[self.daoBooks update:newBook];
	}
	
	[self.tableView reloadData];
	[self.navigationController popViewControllerAnimated:YES];
}

#pragma mark - Private methods

/**
 * 書籍の追加ボタンが押された時に発生します。
 *
 * @param sender 送信者データ。
 */
- (void)addBook:(id)sender
{
	EditBookController* editor = [[EditBookController alloc] initWithNibName:@"EditBook" bundle:nil];
	editor.delegate = self;
	editor.title    = NSLocalizedString( @"BOOK_EDIT_NEW_TITLE", @"" );
	
	UINavigationController* navi = [[UINavigationController alloc] initWithRootViewController:editor];
	navi.navigationBar.barStyle = UIBarStyleBlack;
	
	[self.navigationController presentModalViewController:navi animated:YES];
	[editor release];
	[navi   release];
}

/**
 * 新しい書籍を追加します。
 *
 * @param newBook 書籍。
 */
- (void)addNewBook:(Book*)newBook
{
	NSMutableArray* booksByAuthor = [self.books objectForKey:newBook.author];
	if( booksByAuthor )
	{
		[booksByAuthor addObject:newBook];
	}
	else
	{
		booksByAuthor = [[[NSMutableArray alloc] initWithCapacity:1] autorelease];
		[booksByAuthor addObject:newBook];
		
		[self.books setObject:booksByAuthor forKey:newBook.author];
		[self.authors addObject:newBook.author];
	}
}

/**
 * 指定された位置の書籍を取得します。
 *
 * @param indexPath 位置。
 *
 * @return 書籍。
 */
- (Book*)bookAtIndexPath:(NSIndexPath*)indexPath
{
	NSArray* booksByAuthor = [self.books objectForKey:[self.authors objectAtIndex:indexPath.section]];
	return [booksByAuthor objectAtIndex:indexPath.row];
}

/**
 * 指定された位置の書籍を削除します。
 *
 * @param indexPath 位置。
 */
- (void)removeBook:(NSIndexPath*)indexPath
{
	NSMutableArray* booksByAuthor = [self.books objectForKey:[self.authors objectAtIndex:indexPath.section]];
	
	Book* book = [booksByAuthor objectAtIndex:indexPath.row];
	[self.daoBooks remove:book.bookId];
	
	[self.tableView beginUpdates];
	
	if( booksByAuthor.count == 1 )
	{
		[self.authors removeObject:book.author];
		[self.books removeObjectForKey:book.author];
		[self.tableView deleteSections:[NSIndexSet indexSetWithIndex:indexPath.section] withRowAnimation:UITableViewRowAnimationFade];
	}
	else
	{
		[booksByAuthor removeObjectAtIndex:indexPath.row];
		[self.tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];	
	}
	
	[self.tableView endUpdates];
}

/**
 * 古い書籍を削除します。
 *
 * @param oldBook 書籍。
 */
- (void)removeOldBook:(Book*)oldBook
{
	// 著者が変更されたので、旧リストから除外
	NSMutableArray* booksByAuthor = [self.books objectForKey:oldBook.author];
	for( Book* book in booksByAuthor )
	{
		if( book.bookId == oldBook.bookId )
		{
			[booksByAuthor removeObject:book];
			break;
		}
	}
	
	// 対応する著者の書籍がなくなったなら、著者も削除
	if( booksByAuthor.count == 0 )
	{
		for( NSString* author in self.authors )
		{
			if( [author isEqualToString:oldBook.author] )
			{
				[self.authors removeObject:author];
				break;
			}
		}
	}
}

@end
