//
//  DaoBooks.m
//  TestDataManageer
//
//  Created by Akabeko on 11/11/05.
//  Copyright (c) 2011, Akabeko. All rights reserved.
//

#import "DaoBooks.h"
#import "FMDatabase.h"
#import "FMResultSet.h"
#import "Book.h"

#define DB_FILE_NAME @"app.db"

#define SQL_CREATE @"CREATE TABLE IF NOT EXISTS books (id INTEGER PRIMARY KEY AUTOINCREMENT, author TEXT, title TEXT, copyright INTEGER);"
#define SQL_INSERT @"INSERT INTO books (author, title, copyright) VALUES (?, ?, ?);"
#define SQL_UPDATE @"UPDATE books SET author = ?, title = ?, copyright = ? WHERE id = ?;"
#define SQL_SELECT @"SELECT id, author, title, copyright FROM books GROUP BY author, title;"
#define SQL_DELETE @"DELETE FROM books WHERE id = ?;"

@interface DaoBooks()
@property (nonatomic, copy) NSString* dbPath; //! データベース　ファイルへのパス

- (FMDatabase*)getConnection;
+ (NSString*)getDbFilePath;
@end

@implementation DaoBooks

@synthesize dbPath;

#pragma mark - Lifecycle methods

/**
 * インスタンスを初期化します。
 *
 * @return 初期化後のインスタンス。
 */
- (id)init
{
	self = [super init];
	if( self )
	{
		FMDatabase* db = [self getConnection];
		[db open];
		[db executeUpdate:SQL_CREATE];
		[db close];
	}

	return self;
}

/**
 * メモリを解放します。
 */
- (void)dealloc
{
	self.dbPath = nil;

	[super dealloc];
}

#pragma mark - Public methods

/**
 * 書籍を追加します。
 *
 * @param book 書籍。
 *
 * @return 成功時は識別子を割り当てられた書籍。失敗時は nil。
 */
- (Book*)add:(Book *)book
{
	FMDatabase* db = [self getConnection];
	[db open];

	[db setShouldCacheStatements:YES];
	if( [db executeUpdate:SQL_INSERT, book.author, book.title, book.copyright] )
	{
		book.bookId = [db lastInsertRowId];
	}
	else
	{
		book = nil;
	}
	
	[db close];

	return book;
}

/**
 * 書籍コレクションを取得します。
 *
 * @return 書籍コレクション。
 */
- (NSArray *)books
{
	FMDatabase* db = [self getConnection];
	[db open];
	
	FMResultSet*    results = [db executeQuery:SQL_SELECT];
	NSMutableArray* books = [[[NSMutableArray alloc] initWithCapacity:0] autorelease];
	
	while( [results next] )
	{
		Book* book = [[Book alloc] init];
		book.bookId    = [results intForColumnIndex:0];
		book.author    = [results stringForColumnIndex:1];
		book.title     = [results stringForColumnIndex:2];
		book.copyright = [results dateForColumnIndex:3];
		
		[books addObject:book];
		[book release];
	}
	
	[db close];
	
	return books;
}

/**
 * 書籍を削除します。
 *
 * @param bookId
 *
 * @return 成功時は YES。それ以外は NO。
 */
- (BOOL)remove:(NSInteger)bookId
{
	FMDatabase* db = [self getConnection];
	[db open];

	BOOL isSucceeded = [db executeUpdate:SQL_DELETE, [NSNumber numberWithInteger:bookId]];
	
	[db close];

	return isSucceeded;
}

/**
 * 書籍を更新します。
 */
- (BOOL)update:(Book *)book
{
	FMDatabase* db = [self getConnection];
	[db open];
	
	BOOL isSucceeded = [db executeUpdate:SQL_UPDATE, book.author, book.title, book.copyright, [NSNumber numberWithInteger:book.bookId]];
	
	[db close];
	
	return isSucceeded;
}

#pragma mark - Private methods

/**
 * データベースを取得します。
 *
 * @return データベース。
 */
- (FMDatabase *)getConnection
{
	if( self.dbPath == nil )
	{
		self.dbPath =  [DaoBooks getDbFilePath];
	}
	
	return [FMDatabase databaseWithPath:self.dbPath];
}

/**
 * データベース ファイルのパスを取得します。
 */
+ (NSString*)getDbFilePath
{
	NSArray*  paths = NSSearchPathForDirectoriesInDomains( NSDocumentDirectory, NSUserDomainMask, YES );
	NSString* dir   = [paths objectAtIndex:0];
	
	return [dir stringByAppendingPathComponent:DB_FILE_NAME];
}

@end
